import { ResolvedLayoutConfig } from '@genesis-community/golden-layout';
import { componentType } from './constants';
/**
 * @public
 * Versioned layout config objects. `v` is the version and `c` contains the layout config.
 * @remarks
 * Versioning the layout config in this way allows changes to the config schema while
 * remaining backwards compatible with any existing client/user configurations. When loading an
 * older version of the configurations the version can be used to lookup and apply transformer functions
 * to get the newest version of the config from it.
 *
 * As more versions of configurations are added this type will have extra union types as part of SerialisedLayout.
 */
export type SerialisedLayout = {
    v: '1';
    c: ResolvedLayoutConfig;
};
/**
 * @public
 * The parameters that can be set on a new item when being added by the {@link FoundationLayout.addItem} API
 *
 * @remarks
 * `elements` - array of Elements that are the content of the new item
 * `title` - optional string which is used as the new title of the tab (defaults to be the registered name)
 * `closable` - optional boolean which allows the item to be closed with the X button if set and true
 * `size` - optional string describing the size of the new item (see the written documentation for more info)
 * `registration` - optional string configuring the registration ID of the new item (defaults to sequential IDs)
 */
export interface RegisteredElementConfig {
    registration: string;
    title?: string;
    closable?: boolean;
    size?: string;
}
/**
 * @public
 * Where to and how to add the new item(s) into the layout when using the {@link FoundationLayout.addItem} API.
 * @remarks
 * `area` dictates what part of the layout the new item(s) will be added to.
 * `multiple` is optional and is defaulted to `type=horizontal` and `size=50%` if not defined. `multiple` config only affects the placement if multiple items are specified to be added.
 * `multiple.type` sets the group type of the new items, whether they are horizontal, vertical, or in tabs.
 * `multiple.size` - optional string describing the size of the new item (see the written documentation for more info).
 * `multiple.size` specifies the size of the new group to be added and then the `size` config of the new items are the sizes relative to this new multiple group. If you only specify one new item then the `size` of that will be used as the size and will override `multiple.size`.
 */
export type Placement = {
    area: 'top' | 'left' | 'bottom' | 'right';
    multiple?: {
        type: LayoutRegionType;
        size?: string;
    };
};
/** @internal */
export interface RegistrationConfig {
    elements: Element[];
    id?: string;
}
/** @internal */
export declare const layoutRegionValue: readonly ["horizontal", "vertical", "tabs"];
/**
 * @public
 * Union type describing the three different types of region splits.
 * Set on the `type` attribute on {@link FoundationLayoutRegion}.
 */
export type LayoutRegionType = (typeof layoutRegionValue)[number];
/** @internal */
export type ComponentTypes = LayoutRegionType | 'root' | 'item';
/** @internal */
export interface LayoutComponent {
    [componentType]: ComponentTypes;
    cacheElementsAndRegister(config: RegistrationConfig): string;
    requestLayoutReload(): void;
}
/** @internal */
export type AutosavedLayouts = {
    [x: string]: string;
};
//# sourceMappingURL=types.d.ts.map