var _FoundationLayout__boundDragListener, _a;
import { __classPrivateFieldGet, __classPrivateFieldSet, __decorate } from "tslib";
import { GoldenLayout, LayoutConfig, ResolvedLayoutConfig, } from '@genesis-community/golden-layout';
import { Session } from '@genesislcap/foundation-comms';
import { layoutCacheDocument, UUID } from '@genesislcap/foundation-utils';
import { html, attr, observable, when, ref } from '@microsoft/fast-element';
import { FoundationElement } from '@microsoft/fast-foundation';
import { globalDraggingStyles, glVisualConfig, layoutStyles } from '../styles';
import { DEFAULT_RELOAD_BUFFER, LayoutEmitEvents, LayoutReceiveEvents, componentType, AUTOSAVE_KEY, regionConveter, } from '../utils/';
import { getMissingArrayItems } from '../utils/';
import { LayoutRegistrationError, LayoutUsageError } from '../utils/error';
import { logger } from '../utils/logger';
export { layoutStyles } from '../styles';
/*
 * We only want to apply the global stylesheet required for golden layout
 * to work once, and only apply it if we actually use an instance of the
 * layout that requires it
 */
let appliedGlobalStyles = false;
const defaultAddItemPlacement = {
    area: 'right',
    multiple: {
        type: 'tabs',
        size: '33%',
    },
};
/**
 * @public
 * Top level web component which is used to initialise a custom layout
 * @remarks
 * Can be used in a composition to customise the styles of the layout.
 * @tagname %%prefix%%-layout
 */
export class FoundationLayout extends FoundationElement {
    /** @internal */
    constructor() {
        super();
        this.layoutConfig = { root: undefined };
        /** @internal */
        this[_a] = 'root';
        /**
         * Number describing how long to wait in ms before reloading the config when adding items
         * declaratively by the html API. Default 500 (ms).
         * @public
         */
        this.reloadBuffer = DEFAULT_RELOAD_BUFFER;
        this.reloadPending = false;
        this.registeredComponents = 0;
        /**
         * Function which is used to generate the placeholder text when a layout is loaded
         * with a missing item.
         * @public
         */
        this.missingItemPlaceholder = (missingItem) => `"${missingItem}" cannot be loaded as it isn't registered with the layout system`;
        /**
         * Boolean signifies whether the layout has loaded for the first time or not.
         * @remarks
         * When using the declarative API this is set to true when the layout loads after the first
         * timeout of `reload-buffer`. If using the JavaScript API this occurs after the first call
         * to {@link FoundationLayout.addItem | addItem()}.
         *
         * When using the `LifecycleMixin`, the mixin can be used to gate lifecycle methods from
         * running before the items are inside of the layout.
         * @public
         */
        this.hasFirstLoaded = false;
        /** @internal */
        this.usingDeclerativeAPI = false;
        /**
         * @public
         * Set to true when the user is currently dragging the panes inside of the layout
         */
        this.dragging = false;
        /** @internal */
        _FoundationLayout__boundDragListener.set(this, undefined);
        this.onDragStart = this.onDragStart.bind(this);
        this.onDragStop = this.onDragStop.bind(this);
        this.cacheAndSaveLayout = this.cacheAndSaveLayout.bind(this);
        this.onItemRemoved = this.onItemRemoved.bind(this);
    }
    /** @internal */
    connectedCallback() {
        super.connectedCallback();
        logger.debug(`FoundationLayout connected to the DOM`);
        this.layout = new GoldenLayout(this.layoutElement);
        this.layout.resizeWithContainerAutomatically = true;
        if (!appliedGlobalStyles) {
            new CSSStyleSheet()
                .replace(globalDraggingStyles)
                .then((x) => document.adoptedStyleSheets.push(x))
                .catch((e) => console.error('Failed to replace styles:', e));
            appliedGlobalStyles = true;
        }
        this.shadowRoot.addEventListener('dragStart', this.onDragStart, true);
        this.shadowRoot.addEventListener('dragStop', this.onDragStop, true);
        this.shadowRoot.addEventListener('closeButtonPressed', this.onItemRemoved, true);
    }
    /** @internal */
    disconnectedCallback() {
        this.shadowRoot.removeEventListener('dragStart', this.onDragStart);
        this.shadowRoot.removeEventListener('dragStop', this.onDragStop);
        this.shadowRoot.removeEventListener('closeButtonPressed', this.onItemRemoved);
    }
    /** @internal */
    onDragStart() {
        this.dragging = true;
    }
    /** @internal */
    onDragStop() {
        this.dragging = false;
        this.cacheAndSaveLayout();
        this.attatchResizeEvents();
    }
    /** @internal */
    onItemAdded() {
        this.$emit(LayoutEmitEvents.itemAdded);
        this.cacheAndSaveLayout();
    }
    /** @internal */
    onItemRemoved() {
        this.$emit(LayoutEmitEvents.itemRemoved);
        this.cacheAndSaveLayout();
    }
    /** @internal */
    onItemResized() {
        this.$emit(LayoutEmitEvents.itemResized);
        this.cacheAndSaveLayout();
    }
    /**
     * JS API, public
     */
    /**
     * @public
     * Gets all of the required element registry function names for a set of config
     * @remarks
     * You can use this with {@link FoundationLayout.registeredItems | registeredItems} to work out what items you need to register with {@link FoundationLayout.registerItem | registerItem()}
     * before loading the layout with {@link FoundationLayout.loadLayout | loadLayout()}
     *
     * @param layout - any version of {@link SerialisedLayout} object describing the layout
     * @returns string[] - an item for each registered element in the config. These must all be added before {@link FoundationLayout.loadLayout | function}
     * @throws - various errors if the layout parameter cannot be parsed
     */
    static layoutRequiredRegistrations(layout) {
        const { c: config } = layout;
        const layoutConfig = LayoutConfig.fromResolved(ResolvedLayoutConfig.unminifyConfig(config));
        const names = [];
        JSON.stringify(layoutConfig, (k, v) => {
            if (k === 'componentType') {
                names.push(v);
            }
            return v;
        });
        return names.sort();
    }
    /**
     * @public
     * Gets all of the currently registered names
     * @remarks
     * You can use this with {@link FoundationLayout.layoutRequiredRegistrations | layoutRequiredRegistrations} to work out what items you need to register with {@link FoundationLayout.registerItem | registerItem()}
     * before loading the layout with {@link FoundationLayout.loadLayout | loadLayout()}
     *
     * @returns string[] - an item for each registered element currently registered with the layout system
     */
    registeredItems() {
        return this.layout.getRegisteredComponentTypeNames();
    }
    /**
     * @public
     * Gets a minified string containing the config describing the current layout of the layout object to later restore in {@link FoundationLayout.loadLayout | function}
     * @returns - latest version of {@link SerialisedLayout} describing the layout
     */
    getLayout() {
        return { v: '1', c: ResolvedLayoutConfig.minifyConfig(this.layout.saveLayout()) };
    }
    /**
     * Try to load a layout from local storage, or return false. Only required if manually calling {@link FoundationLayout.registerItem}
     * @remarks
     * Attempt to load an autosaved layout from local storage, keyed on the `auto-save-key` attribute.
     * If `auto-save-key` attribute is not set or there is no autosaved layout yet, this will return false. Else, true.
     *
     * This function is automatically called when loading the layout via the declarative API so if you're not registering components via
     * the JavaScript API then you don't need to call this function. If you *are* calling {@link FoundationLayout.registerItem} then you should call this function immediately afterwards.
     *
     * Will load the layout with `handleMissingItem = 'placeholder` so placeholder text will be shown for any missing items.
     * @returns boolean - true if a layout was loaded, false if not
     * @public
     */
    tryLoadLayoutFromLocalStorage() {
        if (!this.autoSaveKey)
            return false;
        const existingLayouts = JSON.parse(this.session.getLocalStorageItem(AUTOSAVE_KEY) || '{}');
        if (!(this.autoSaveKey in existingLayouts))
            return false;
        const layout = JSON.parse(existingLayouts[this.autoSaveKey]);
        this.loadLayout(layout, 'placeholder');
        return true;
    }
    /**
     * @public
     * Restores a layout described in the config from {@link FoundationLayout.getLayout | getLayout()}
     *
     * @remarks
     * In order to restore a layout you must have registered all of the required panes with the layout system before restoring it. If you are just setting the layout using the declarative API then all of the same components will be registered. If you have added any elements using {@link FoundationLayout.registerItem} then you must ensure all of the same items have been added again. To make tracking what items are registered easier it is recommended in this case you explicitly name your registrations using the `id` parameter available on the {@link RegisteredElementConfig} and `foundation-layout-item` APIs. You can use {@link FoundationLayout.layoutRequiredRegistrations} to check which items are registered in a current layout in order to dynamically add any missing items before you can restore the layout
     *
     * @param layout - any version of {@link SerialisedLayout} object describing the layout
     * @param handleMissingItem - what to do if the layout contains items that are not currently registered with the layout system. Defaults to 'error' which will throw an error. If set to 'placeholder' then any missing items will be replaced with a placeholder element. You can control the text of the placeholder element with {@link FoundationLayout.missingItemPlaceholder}.
     * @throws {@link LayoutUsageError} if you attempt to load a layout with registered items that are not currently registered with the layout system, and handleMissingItem is set to 'error' (default).
     * @throws various errors if the layout string is malformed and cannot be parsed
     */
    loadLayout(layout, handleMissingItem = 'error') {
        const alreadyRegistered = this.registeredItems();
        const wantedRegistered = FoundationLayout.layoutRequiredRegistrations(layout);
        const missingRegisteredItems = getMissingArrayItems(wantedRegistered, alreadyRegistered);
        if (missingRegisteredItems.length !== 0 && handleMissingItem === 'error') {
            throw new LayoutUsageError(`Trying to load layout with extra components. The component(s) not currently loaded are "${[
                ...new Set(missingRegisteredItems),
            ]}"`);
        }
        const layoutConfig = LayoutConfig.fromResolved(ResolvedLayoutConfig.unminifyConfig(layout.c));
        if (missingRegisteredItems.length !== 0 && handleMissingItem === 'placeholder') {
            this.registerPlaceholdersAndSetClosable(layoutConfig, missingRegisteredItems);
        }
        this.loadGLConfigAndSetup(layoutConfig);
        this.cacheAndSaveLayout();
    }
    /**
     * @public
     * Dynamically add a new item to the layout. The user can move the new plane to whenever they want once it has been added.
     * @remarks
     * Adding new items invokes the registration previously made explicitly via {@link FoundationLayout.registerItem | registerItem()} or implicitly via the html declarative API.
     *
     * The elements added onto the new pane are copies using `element.cloneNode()` of the original element references used during registration.
     *
     * @param config - {@link RegisteredElementConfig} configuration items for the new items(s). Pass an array of {@link RegisteredElementConfig} to add multiple items at once.
     * @param placement - where and how to add the new items to the layout. For more info and defaults see {@link Placement}.
     * @throws {@link LayoutRegistrationError} if you attempt to add an item before it has been registered
     */
    addItem(config, placement = defaultAddItemPlacement) {
        var _b, _c;
        const configArray = Array.isArray(config) ? config : [config];
        const isSingleItem = configArray.length === 1;
        const childItems = configArray.map((regConfig) => {
            const { registration, title, size, closable } = regConfig;
            if (!this.layout.getRegisteredComponentTypeNames().includes(registration)) {
                throw new LayoutRegistrationError(`Cannot add item '${registration}' which is not registered with the layout system.`);
            }
            return {
                type: 'component',
                componentType: registration,
                title: title !== null && title !== void 0 ? title : `Item ${registration}`,
                reorderEnabled: true,
                isClosable: closable || false,
                size: !isSingleItem ? size : undefined,
            };
        });
        const itemConfig = {
            type: regionConveter(((_b = placement === null || placement === void 0 ? void 0 : placement.multiple) === null || _b === void 0 ? void 0 : _b.type) || 'tabs'),
            content: childItems,
            size: isSingleItem ? configArray[0].size : (_c = placement === null || placement === void 0 ? void 0 : placement.multiple) === null || _c === void 0 ? void 0 : _c.size,
        };
        /**
         * If no root is added (no items via declerative API) then
         * this new item is the root so set it and return
         */
        if (typeof this.layoutConfig.root === 'undefined') {
            this.layoutConfig = {
                root: itemConfig,
            };
            this.loadGLConfigAndSetup(this.layoutConfig);
            this.onItemAdded();
            return;
        }
        // Else we add the item relative to existing items
        this.cacheAndSaveLayout();
        const { area } = placement;
        this.layoutConfig = {
            root: {
                type: area === 'top' || area === 'bottom' ? 'column' : 'row',
                content: area === 'right' || area === 'bottom'
                    ? [this.layoutConfig.root, itemConfig]
                    : [itemConfig, this.layoutConfig.root],
            },
        };
        this.loadGLConfigAndSetup(this.layoutConfig);
        this.onItemAdded();
    }
    /**
     * @public
     * Register a collection of `Element` and associate them with an `ID` with the layout system for later use.
     * @remarks
     * You would use this to register elements that you later want to load when using {@link FoundationLayout.loadLayout}.
     * Use {@link FoundationLayout.layoutRequiredRegistrations} to see what components need to be registered for a certain config
     * and then register them using this function before calling {@link FoundationLayout.loadLayout}.
     *
     * When registering an element it is moved by reference into the internals of the layout, so if you pass elements already in the DOM then they will disappear. If you want to avoid this you can pass copies using `element.cloneNode(true)`.
     *
     * @param registration - string of the registration ID
     * @param elements - Elements[] containing the reference to the elements to register for later usage
     * @throws {@link LayoutUsageError} if you attempt to add an item before the layout has been initialised.
     * @throws {@link LayoutRegistrationError} if you attempt to use a `registration` name which is already in use (declarative html API and JavaScript API registrations use the same "pool" of registration names).
     * @returns - string defining the name of the registered item with the layout system (config.id if set).
     */
    registerItem(registration, elements) {
        if (!this.layout.isInitialised) {
            throw new LayoutUsageError('Cannot add item via JS API until initialised');
        }
        return this.cacheElementsAndRegister({
            elements: elements,
            id: registration,
        });
    }
    /**
     * Internal APIs
     */
    /**
     * Request to reload the layout using the private member config. Debounced using the time set
     * in this.reloadBuffer
     * @internal
     */
    requestLayoutReload() {
        if (this.reloadPending)
            return;
        this.reloadPending = true;
        setTimeout(() => {
            this.reloadPending = false;
            if (!this.hasFirstLoaded) {
                this.$emit(LayoutEmitEvents.firstLoaded);
                this.hasFirstLoaded = true;
                const res = this.tryLoadLayoutFromLocalStorage();
                if (!res) {
                    this.loadGLConfigAndSetup(this.layoutConfig);
                }
            }
            else {
                this.loadGLConfigAndSetup(this.layoutConfig);
            }
        }, this.reloadBuffer);
    }
    /**
     * Adds a config item into the root config and returns a link to the new config in the tree
     * @typeParam T - RowOrColumnItemConfig | StackItemConfig | ComponentItemConfig;
     * @param item - T to add as the root element of the layout
     * @returns T the reference to the added item in the whole item config
     * @throws {@link LayoutUsageError} if you try and add an item when a root has already been set (this happens if you have multiple direct children of this html element which is an incorrect usage)
     * @internal
     */
    addItemFromChild(item) {
        if (typeof this.layoutConfig.root !== 'undefined') {
            throw new LayoutUsageError('Cannot have multiple layout items as direct descendants of the layout root. You need to put the layout items within a layout region.');
        }
        this.usingDeclerativeAPI = true;
        this.layoutConfig = {
            root: item,
        };
        return this.layoutConfig.root;
    }
    /**
     * Registers a function with golden layout to create a pane
     * @param elements - Elements[] to add to new new pane
     * @param id - optional string which is used to register the new function with golden layout. Defaults to sequentially setting the IDs for default items
     * @returns - string which is the registered ID
     * @throws - {@link LayoutRegistrationError} if the id is already in use
     * @internal
     */
    cacheElementsAndRegister({ elements, id }) {
        const reg = id || `${(this.registeredComponents += 1)}`;
        if (this.layout.getRegisteredComponentTypeNames().includes(reg)) {
            throw new LayoutRegistrationError(`Cannot register item with already registered name: '${reg}'`);
        }
        /**
         * What we need the element registry function to do:
         * "Consume" the registered elements initially, so we don't have duplicate items from the declarative API
         * We need to hold a master copy of the registered items
         * When an item is created at first, it is copied from the master copy so it behaves as expected
         * When a copy is created it becomes its own instance for the lifetime of the layout in the DOM
         *
         * This has the effect of allowing us to create multiple instances of the same registered item, and each
         * created instance is separate allowing it to implement its own serialise and cache of data.
         */
        const registrationFunction = (() => {
            // Use appendChild to consume the elements and save them in the master copy
            const masterCopy = document.createDocumentFragment();
            masterCopy[layoutCacheDocument] = true;
            elements.forEach((e) => masterCopy.appendChild(e));
            const instances = new Map();
            return (container, state) => {
                var _b;
                // If this is a new instance then assign it uuid instance
                if (!(state === null || state === void 0 ? void 0 : state['instance'])) {
                    state['instance'] = this.uuid.createId();
                }
                container.state['instance'] = state['instance'];
                container.state['originalTitle'] = (_b = state['originalTitle']) !== null && _b !== void 0 ? _b : container.title;
                // If this is a new instance then copy the master copy into the instances map
                // this is then the instance that is recalled for this version each time
                // the key point is "cloneNode" which makes a copy at this point
                if (!instances.has(state === null || state === void 0 ? void 0 : state['instance'])) {
                    const instanceCopy = document.createDocumentFragment();
                    Array.from(masterCopy.children).forEach((e) => instanceCopy.appendChild(e.cloneNode(true)));
                    instances.set(state['instance'], [...instanceCopy.children]);
                }
                // get the instance from the map and append it to the container
                instances.get(state['instance']).forEach((e) => container.element.appendChild(e));
                this.setupLayoutReceiveEvents(container, state);
            };
        })();
        this.layout.registerComponentFactoryFunction(reg, registrationFunction);
        return reg;
    }
    /**
     * Sets up the event listeners for the layout receive events
     * @internal
     */
    setupLayoutReceiveEvents(container, state) {
        const originalTitle = container.state['originalTitle'];
        container.element.addEventListener(LayoutReceiveEvents.changeTitle, (e) => {
            const maybeDetail = e === null || e === void 0 ? void 0 : e.detail;
            if (!((maybeDetail === null || maybeDetail === void 0 ? void 0 : maybeDetail.title) && (maybeDetail === null || maybeDetail === void 0 ? void 0 : maybeDetail.mode))) {
                logger.error('Invalid change-title event.detail:', e === null || e === void 0 ? void 0 : e.detail);
            }
            container.setTitle(`${maybeDetail.mode === 'suffix' ? originalTitle + ' ' : ''}${maybeDetail.title}`);
        });
    }
    /**
     * Saves the stored layout config according to the Golden Layout provider.
     * And caches the layout in local storage if {@link FoundationLayout.autoSaveKey} is set.
     * @internal
     * */
    cacheAndSaveLayout() {
        this.layoutConfig = LayoutConfig.fromResolved(this.layout.saveLayout());
        if (this.autoSaveKey) {
            const existingLayouts = JSON.parse(this.session.getLocalStorageItem(AUTOSAVE_KEY) || '{}');
            existingLayouts[this.autoSaveKey] = JSON.stringify(this.getLayout());
            this.session.setLocalStorageItem(AUTOSAVE_KEY, JSON.stringify(existingLayouts));
        }
    }
    /**
     * Used when we are trying to load a layout with missing registrations and we want to add placeholder items
     * @remarks
     * This function will register the items with placeholder text, and set them to be closeable if they were not already
     * As config is passed by reference it is updated directly
     * @param config - The layout config to update
     * @param missingRegisteredItems - The items which were missing from the registrations
     * @internal
     */
    registerPlaceholdersAndSetClosable(config, missingRegisteredItems) {
        const placeholderElement = document.createElement('p');
        placeholderElement.style.textAlign = 'center';
        placeholderElement.style.position = 'relative';
        placeholderElement.style.top = '50%';
        const recurseSetClosable = (node) => {
            var _b;
            if (node.type === 'component' &&
                missingRegisteredItems.includes(node.componentType.toString())) {
                const nodeName = node.componentType.toString();
                node.isClosable = true;
                const placeholderCopy = placeholderElement.cloneNode();
                placeholderCopy.textContent = this.missingItemPlaceholder(nodeName);
                this.registerItem(nodeName, [placeholderCopy]);
            }
            (_b = node.content) === null || _b === void 0 ? void 0 : _b.forEach(recurseSetClosable);
        };
        recurseSetClosable(config.root);
    }
    /**
     * Loads the provided config into Golden Layout and does other setup
     * @remarks
     * Loads the config into Golden Layout, and then sets up the event listeners for the layout
     * When loading the layout the visual configuration is loaded
     * Event listeners are added to the drag handles to emulate resizing events
     *
     * **This is the only function which should call this.layout.loadLayout() directly.**
     * @internal
     */
    loadGLConfigAndSetup(config) {
        this.hasFirstLoaded = true;
        this.layout.loadLayout(Object.assign(Object.assign({}, config), glVisualConfig));
        this.attatchResizeEvents();
    }
    /**
     * Handles adding the drag event listeners onto the golden layout drag handles
     * @remarks
     * If we only add the event listeners once then once the user drags items around the layout and creates
     * new drag handles, they will not have the event listeners attached. This function will add the event listeners
     * but also ensure that old listeners are cleaned up. To be able to clean up the event we need a reference
     * rather than an anonymous function so we store the bound function in a private variable.
     * @internal
     */
    attatchResizeEvents() {
        if (__classPrivateFieldGet(this, _FoundationLayout__boundDragListener, "f") === undefined) {
            __classPrivateFieldSet(this, _FoundationLayout__boundDragListener, () => this.onItemResized.bind(this)(), "f");
        }
        this.shadowRoot.querySelectorAll('div.lm_drag_handle').forEach((h) => {
            h.removeEventListener('click', __classPrivateFieldGet(this, _FoundationLayout__boundDragListener, "f"));
            h.addEventListener('click', __classPrivateFieldGet(this, _FoundationLayout__boundDragListener, "f"));
        });
    }
}
_FoundationLayout__boundDragListener = new WeakMap(), _a = componentType;
__decorate([
    attr({ attribute: 'reload-buffer' })
], FoundationLayout.prototype, "reloadBuffer", void 0);
__decorate([
    UUID
], FoundationLayout.prototype, "uuid", void 0);
__decorate([
    Session
], FoundationLayout.prototype, "session", void 0);
__decorate([
    attr({ attribute: 'auto-save-key' })
], FoundationLayout.prototype, "autoSaveKey", void 0);
__decorate([
    observable
], FoundationLayout.prototype, "missingItemPlaceholder", void 0);
__decorate([
    observable
], FoundationLayout.prototype, "hasFirstLoaded", void 0);
__decorate([
    observable
], FoundationLayout.prototype, "usingDeclerativeAPI", void 0);
__decorate([
    observable
], FoundationLayout.prototype, "dragging", void 0);
const loadingTemplate = html `
  <div class="html-spinner"></div>
`;
/**
 * `ViewTemplate` which defines the html for {@link FoundationLayout}.
 * @remarks
 * Can be used in a composition to customise the styles of the layout.
 * @public
 */
export const layoutTemplate = html `
  <template>
    ${when((x) => !x.hasFirstLoaded && x.usingDeclerativeAPI, loadingTemplate)}
    <div class="layout-container" ${ref('layoutElement')}></div>
  </template>
`;
/**
 * The Foundation Layout Composed element
 *
 * @internal
 * @remarks
 * Can be registered with the design system via {@link foundationlayoutComponents}.
 *
 * Tag Name: <{prefix}-layout>
 */
export const foundationLayout = FoundationLayout.compose({
    baseName: 'layout',
    styles: layoutStyles,
    template: layoutTemplate,
});
