import type { Container } from '@microsoft/fast-foundation';
import { ElementStyles } from '@microsoft/fast-element';
import { FoundationElement } from '@microsoft/fast-foundation';
import { FoundationElementRegistry } from '@microsoft/fast-foundation';
import { OverrideFoundationElementDefinition } from '@microsoft/fast-foundation';
import { ResolvedLayoutConfig } from '@genesis-community/golden-layout';
import { RootItemConfig } from '@genesis-community/golden-layout';
import { ViewTemplate } from '@microsoft/fast-element';

/**
 * Used to key what type of LayoutComponent an object is
 * @internal
 */
declare const componentType: unique symbol;

/** @internal */
declare type ComponentTypes = LayoutRegionType | 'root' | 'item';

/**
 * Default time in milliseconds for the layout to buffer calls to reloading
 * the layout while the declarative API is loading.
 *
 * During the first load of the layout, a loading spinner will be shown.
 * @public
 */
export declare const DEFAULT_RELOAD_BUFFER = 500;

/**
 * @public
 * Top level web component which is used to initialise a custom layout
 * @remarks
 * Can be used in a composition to customise the styles of the layout.
 * @tagname %%prefix%%-layout
 */
export declare class FoundationLayout extends FoundationElement implements LayoutComponent {
    #private;
    private layout;
    /** @internal */
    layoutElement: HTMLElement;
    private layoutConfig;
    /** @internal */
    [componentType]: "root";
    /**
     * Number describing how long to wait in ms before reloading the config when adding items
     * declaratively by the html API. Default 500 (ms).
     * @public
     */
    reloadBuffer: number;
    private reloadPending;
    private registeredComponents;
    /** @internal */
    private uuid;
    /** @internal */
    private session;
    /**
     * Attribute which if set will auto save and load the layout as the user changes it.
     * Omit this attribute to disable this feature.
     * Set attribute using `auto-save-key`.
     * @public
     */
    autoSaveKey?: string;
    /**
     * Function which is used to generate the placeholder text when a layout is loaded
     * with a missing item.
     * @public
     */
    missingItemPlaceholder: (missingItem: string) => string;
    /**
     * Boolean signifies whether the layout has loaded for the first time or not.
     * @remarks
     * When using the declarative API this is set to true when the layout loads after the first
     * timeout of `reload-buffer`. If using the JavaScript API this occurs after the first call
     * to {@link FoundationLayout.addItem | addItem()}.
     *
     * When using the `LifecycleMixin`, the mixin can be used to gate lifecycle methods from
     * running before the items are inside of the layout.
     * @public
     */
    hasFirstLoaded: boolean;
    /** @internal */
    usingDeclerativeAPI: boolean;
    /**
     * @public
     * Set to true when the user is currently dragging the panes inside of the layout
     */
    dragging: boolean;
    /** @internal */
    constructor();
    /** @internal */
    connectedCallback(): void;
    /** @internal */
    disconnectedCallback(): void;
    /** @internal */
    private onDragStart;
    /** @internal */
    private onDragStop;
    /** @internal */
    private onItemAdded;
    /** @internal */
    private onItemRemoved;
    /** @internal */
    private onItemResized;
    /**
     * JS API, public
     */
    /**
     * @public
     * Gets all of the required element registry function names for a set of config
     * @remarks
     * You can use this with {@link FoundationLayout.registeredItems | registeredItems} to work out what items you need to register with {@link FoundationLayout.registerItem | registerItem()}
     * before loading the layout with {@link FoundationLayout.loadLayout | loadLayout()}
     *
     * @param layout - any version of {@link SerialisedLayout} object describing the layout
     * @returns string[] - an item for each registered element in the config. These must all be added before {@link FoundationLayout.loadLayout | function}
     * @throws - various errors if the layout parameter cannot be parsed
     */
    static layoutRequiredRegistrations(layout: SerialisedLayout): string[];
    /**
     * @public
     * Gets all of the currently registered names
     * @remarks
     * You can use this with {@link FoundationLayout.layoutRequiredRegistrations | layoutRequiredRegistrations} to work out what items you need to register with {@link FoundationLayout.registerItem | registerItem()}
     * before loading the layout with {@link FoundationLayout.loadLayout | loadLayout()}
     *
     * @returns string[] - an item for each registered element currently registered with the layout system
     */
    registeredItems(): string[];
    /**
     * @public
     * Gets a minified string containing the config describing the current layout of the layout object to later restore in {@link FoundationLayout.loadLayout | function}
     * @returns - latest version of {@link SerialisedLayout} describing the layout
     */
    getLayout(): SerialisedLayout;
    /**
     * Try to load a layout from local storage, or return false. Only required if manually calling {@link FoundationLayout.registerItem}
     * @remarks
     * Attempt to load an autosaved layout from local storage, keyed on the `auto-save-key` attribute.
     * If `auto-save-key` attribute is not set or there is no autosaved layout yet, this will return false. Else, true.
     *
     * This function is automatically called when loading the layout via the declarative API so if you're not registering components via
     * the JavaScript API then you don't need to call this function. If you *are* calling {@link FoundationLayout.registerItem} then you should call this function immediately afterwards.
     *
     * Will load the layout with `handleMissingItem = 'placeholder` so placeholder text will be shown for any missing items.
     * @returns boolean - true if a layout was loaded, false if not
     * @public
     */
    tryLoadLayoutFromLocalStorage(): boolean;
    /**
     * @public
     * Restores a layout described in the config from {@link FoundationLayout.getLayout | getLayout()}
     *
     * @remarks
     * In order to restore a layout you must have registered all of the required panes with the layout system before restoring it. If you are just setting the layout using the declarative API then all of the same components will be registered. If you have added any elements using {@link FoundationLayout.registerItem} then you must ensure all of the same items have been added again. To make tracking what items are registered easier it is recommended in this case you explicitly name your registrations using the `id` parameter available on the {@link RegisteredElementConfig} and `foundation-layout-item` APIs. You can use {@link FoundationLayout.layoutRequiredRegistrations} to check which items are registered in a current layout in order to dynamically add any missing items before you can restore the layout
     *
     * @param layout - any version of {@link SerialisedLayout} object describing the layout
     * @param handleMissingItem - what to do if the layout contains items that are not currently registered with the layout system. Defaults to 'error' which will throw an error. If set to 'placeholder' then any missing items will be replaced with a placeholder element. You can control the text of the placeholder element with {@link FoundationLayout.missingItemPlaceholder}.
     * @throws {@link LayoutUsageError} if you attempt to load a layout with registered items that are not currently registered with the layout system, and handleMissingItem is set to 'error' (default).
     * @throws various errors if the layout string is malformed and cannot be parsed
     */
    loadLayout(layout: SerialisedLayout, handleMissingItem?: 'placeholder' | 'error'): void;
    /**
     * @public
     * Dynamically add a new item to the layout. The user can move the new plane to whenever they want once it has been added.
     * @remarks
     * Adding new items invokes the registration previously made explicitly via {@link FoundationLayout.registerItem | registerItem()} or implicitly via the html declarative API.
     *
     * The elements added onto the new pane are copies using `element.cloneNode()` of the original element references used during registration.
     *
     * @param config - {@link RegisteredElementConfig} configuration items for the new items(s). Pass an array of {@link RegisteredElementConfig} to add multiple items at once.
     * @param placement - where and how to add the new items to the layout. For more info and defaults see {@link Placement}.
     * @throws {@link LayoutRegistrationError} if you attempt to add an item before it has been registered
     */
    addItem(config: RegisteredElementConfig | RegisteredElementConfig[], placement?: Placement): void;
    /**
     * @public
     * Register a collection of `Element` and associate them with an `ID` with the layout system for later use.
     * @remarks
     * You would use this to register elements that you later want to load when using {@link FoundationLayout.loadLayout}.
     * Use {@link FoundationLayout.layoutRequiredRegistrations} to see what components need to be registered for a certain config
     * and then register them using this function before calling {@link FoundationLayout.loadLayout}.
     *
     * When registering an element it is moved by reference into the internals of the layout, so if you pass elements already in the DOM then they will disappear. If you want to avoid this you can pass copies using `element.cloneNode(true)`.
     *
     * @param registration - string of the registration ID
     * @param elements - Elements[] containing the reference to the elements to register for later usage
     * @throws {@link LayoutUsageError} if you attempt to add an item before the layout has been initialised.
     * @throws {@link LayoutRegistrationError} if you attempt to use a `registration` name which is already in use (declarative html API and JavaScript API registrations use the same "pool" of registration names).
     * @returns - string defining the name of the registered item with the layout system (config.id if set).
     */
    registerItem(registration: string, elements: Element[]): string;
    /**
     * Internal APIs
     */
    /**
     * Request to reload the layout using the private member config. Debounced using the time set
     * in this.reloadBuffer
     * @internal
     */
    requestLayoutReload(): void;
    /**
     * Adds a config item into the root config and returns a link to the new config in the tree
     * @typeParam T - RowOrColumnItemConfig | StackItemConfig | ComponentItemConfig;
     * @param item - T to add as the root element of the layout
     * @returns T the reference to the added item in the whole item config
     * @throws {@link LayoutUsageError} if you try and add an item when a root has already been set (this happens if you have multiple direct children of this html element which is an incorrect usage)
     * @internal
     */
    addItemFromChild<T extends RootItemConfig>(item: T): T;
    /**
     * Registers a function with golden layout to create a pane
     * @param elements - Elements[] to add to new new pane
     * @param id - optional string which is used to register the new function with golden layout. Defaults to sequentially setting the IDs for default items
     * @returns - string which is the registered ID
     * @throws - {@link LayoutRegistrationError} if the id is already in use
     * @internal
     */
    cacheElementsAndRegister({ elements, id }: RegistrationConfig): string;
    /**
     * Sets up the event listeners for the layout receive events
     * @internal
     */
    private setupLayoutReceiveEvents;
    /**
     * Saves the stored layout config according to the Golden Layout provider.
     * And caches the layout in local storage if {@link FoundationLayout.autoSaveKey} is set.
     * @internal
     * */
    private cacheAndSaveLayout;
    /**
     * Used when we are trying to load a layout with missing registrations and we want to add placeholder items
     * @remarks
     * This function will register the items with placeholder text, and set them to be closeable if they were not already
     * As config is passed by reference it is updated directly
     * @param config - The layout config to update
     * @param missingRegisteredItems - The items which were missing from the registrations
     * @internal
     */
    private registerPlaceholdersAndSetClosable;
    /**
     * Loads the provided config into Golden Layout and does other setup
     * @remarks
     * Loads the config into Golden Layout, and then sets up the event listeners for the layout
     * When loading the layout the visual configuration is loaded
     * Event listeners are added to the drag handles to emulate resizing events
     *
     * **This is the only function which should call this.layout.loadLayout() directly.**
     * @internal
     */
    private loadGLConfigAndSetup;
    /**
     * Handles adding the drag event listeners onto the golden layout drag handles
     * @remarks
     * If we only add the event listeners once then once the user drags items around the layout and creates
     * new drag handles, they will not have the event listeners attached. This function will add the event listeners
     * but also ensure that old listeners are cleaned up. To be able to clean up the event we need a reference
     * rather than an anonymous function so we store the bound function in a private variable.
     * @internal
     */
    private attatchResizeEvents;
}

/**
 * Registration object to register the layout with your design system.
 *
 * @remarks
 * Registers the three layout component types with the design system.
 * You require to use the prefix of the design system with each tag.
 * If you wish to alternate the styles of the layout you only need to compose your own custom
 * version of {@link FoundationLayout} as this contains the styles for the entire layout.
 *
 * @example
 * `<zero-layout></zero-layout>` if you are using the layout with the `zero` design-system.
 *
 * @public
 */
export declare const foundationLayoutComponents: {
    foundationLayout: (overrideDefinition?: OverrideFoundationElementDefinition<    {
    baseName: string;
    styles: ElementStyles;
    template: ViewTemplate<FoundationLayout, any>;
    }>) => FoundationElementRegistry<    {
    baseName: string;
    styles: ElementStyles;
    template: ViewTemplate<FoundationLayout, any>;
    }, FoundationLayout>;
    foundationLayoutRegion: (overrideDefinition?: OverrideFoundationElementDefinition<    {
    baseName: string;
    template: ViewTemplate<any, any>;
    }>) => FoundationElementRegistry<    {
    baseName: string;
    template: ViewTemplate<any, any>;
    }, FoundationLayoutRegion>;
    foundationLayoutItem: (overrideDefinition?: OverrideFoundationElementDefinition<    {
    baseName: string;
    template: ViewTemplate<any, any>;
    }>) => FoundationElementRegistry<    {
    baseName: string;
    template: ViewTemplate<any, any>;
    }, FoundationLayoutItem>;
    register(container?: Container, ...rest: any[]): void;
};

/**
 * @public
 * `FoundationLayoutItem` is a custom element that represents an item in the layout.
 * @remarks
 * This element is used to wrap html elements and configure their layout settings as part of the layout system.
 *
 * This is a simple component which is only used to define the layout splits; any JavaScript API interactions or custom styling is used via {@link FoundationLayout}.
 * @tagname %%prefix%%-layout-item
 */
export declare class FoundationLayoutItem extends FoundationElement implements LayoutComponent {
    /**
     * Sets the title of the item which is displayed on the tab.
     * @remarks
     * Defaults to `Item ` + {@link FoundationLayoutItem.registration}
     * @public
     */
    title: string;
    /**
     * Boolean attribute controls whether the window can be closed in the GUI.
     * Defaults to `false`.
     * @public
     */
    closable: boolean;
    /**
     * optional string describing the size of the new item (see the written documentation for more info)
     * @public
     */
    size: string;
    /**
     * Sets the registration name for the item, which can be used later to add the item via the JavaScript API using {@link FoundationLayout.addItem}.
     * @remarks
     * Items added via the JavaScript API and HTML API share the same pool of registration names.
     * Using a duplicate registration name is a runtime error.
     * This registration name defaults to the number of the window it is.
     * It is highly recommended if you are using the JavaScript API that you set a registration name here manually.
     * @public
     */
    registration: string;
    /** @internal */
    [componentType]: "item";
    /** @internal */
    private slottedElements;
    /** @internal */
    connectedCallback(): void;
    /** @internal */
    private getParentLayoutComponent;
    /** @internal */
    requestLayoutReload(): void;
    /** @internal */
    cacheElementsAndRegister(config: RegistrationConfig): string;
}

/**
 * @public
 * `FoundationLayoutRegion` is a custom element that represents a region in the layout.
 * @remarks
 * This element is used to create a region in the layout. It can be used to create a horizontal or vertical split, or a tabbed region.
 *
 * This is a simple component which is only used to define the layout splits; any JavaScript API interactions or custom styling is used via {@link FoundationLayout}.
 * @tagname %%prefix%%-layout-region
 */
export declare class FoundationLayoutRegion extends FoundationElement implements LayoutComponent {
    /**
     * optional string describing the size of the new item (see the written documentation for more info)
     * @public
     */
    size: string;
    /**
     * Defines the {@link LayoutRegionType | type} of the region.
     * Defaults to `horizontal`.
     * @public
     */
    type: LayoutRegionType;
    /** @internal */
    [componentType]: LayoutRegionType;
    /**
     * Reference to the config in the whole layout config for this
     * specific region.
     * @internal
     */
    private layoutConfig;
    /** @internal */
    connectedCallback(): void;
    /**
     * Gets the layout config for this region using {@link FoundationLayoutRegion.constructLayoutConfig} and adds it into the main config tree via the parents. It saves a reference to the this config so a child can add its config in later.
     * @internal
     */
    private setupLayoutRegion;
    /**
     * Creates a golden layout config for this object.
     * @internal
     */
    private constructLayoutConfig;
    /** @internal */
    private getParentLayoutComponent;
    /** @internal */
    addItemFromChild<T extends RootItemConfig>(item: T): T;
    /** @internal */
    requestLayoutReload(): void;
    /** @internal */
    cacheElementsAndRegister(config: RegistrationConfig): string;
}

/** @internal */
declare interface LayoutComponent {
    [componentType]: ComponentTypes;
    cacheElementsAndRegister(config: RegistrationConfig): string;
    requestLayoutReload(): void;
}

/**
 * Defines events that the layout system emits
 *
 * 'firstLoaded' - emitted when the layout has finished loading the first time
 * using the declarative API after {@link DEFAULT_RELOAD_BUFFER} ms.
 * <br/>
 * 'itemAdded' - emitted when an item is added to the layout'
 * <br/>
 * 'itemRemoved' - emitted when an item is removed from the layout'
 * <br/>
 * 'itemResized' - emitted when the user drags the divider to resize elements
 * @public
 */
export declare const LayoutEmitEvents: {
    readonly firstLoaded: "first-loaded";
    readonly itemAdded: "item-added";
    readonly itemRemoved: "item-removed";
    readonly itemResized: "item-resized";
};

/**
 * Defines events that the layout system listens for
 *
 * 'changeTitle' - emit this from a contained item to update the title of the window that contains it.
 * @public
 */
export declare const LayoutReceiveEvents: {
    readonly changeTitle: "change-title";
};

/**
 * Defines the shape of the detail that the layout listens works with for events it listens on
 *
 * 'changeTitle' - `title` is the string you want to set. For `mode`: `replace` will set the title to be `title`, `suffix` will append `title` to the end of the existing title.
 * @public
 */
export declare type LayoutReceiveEventsDetail = {
    changeTitle: {
        title: string;
        mode: 'replace' | 'suffix';
    };
};

/**
 * @public
 * Union type describing the three different types of region splits.
 * Set on the `type` attribute on {@link FoundationLayoutRegion}.
 */
export declare type LayoutRegionType = (typeof layoutRegionValue)[number];

/** @internal */
declare const layoutRegionValue: readonly ["horizontal", "vertical", "tabs"];

/**
 * @public
 */
export declare class LayoutRegistrationError extends Error {
    constructor(message: string);
}

/**
 * `ElementStyles` which defines the css for {@link FoundationLayout}.
 * @remarks
 * Can be used in a composition to customise the styles of the layout.
 * @public
 */
export declare const layoutStyles: ElementStyles;

/**
 * `ViewTemplate` which defines the html for {@link FoundationLayout}.
 * @remarks
 * Can be used in a composition to customise the styles of the layout.
 * @public
 */
export declare const layoutTemplate: ViewTemplate<FoundationLayout, any>;

/**
 * @public
 */
export declare class LayoutUsageError extends Error {
    constructor(message: string);
}

/**
 * @public
 * Where to and how to add the new item(s) into the layout when using the {@link FoundationLayout.addItem} API.
 * @remarks
 * `area` dictates what part of the layout the new item(s) will be added to.
 * `multiple` is optional and is defaulted to `type=horizontal` and `size=50%` if not defined. `multiple` config only affects the placement if multiple items are specified to be added.
 * `multiple.type` sets the group type of the new items, whether they are horizontal, vertical, or in tabs.
 * `multiple.size` - optional string describing the size of the new item (see the written documentation for more info).
 * `multiple.size` specifies the size of the new group to be added and then the `size` config of the new items are the sizes relative to this new multiple group. If you only specify one new item then the `size` of that will be used as the size and will override `multiple.size`.
 */
export declare type Placement = {
    area: 'top' | 'left' | 'bottom' | 'right';
    multiple?: {
        type: LayoutRegionType;
        size?: string;
    };
};

/**
 * @public
 * The parameters that can be set on a new item when being added by the {@link FoundationLayout.addItem} API
 *
 * @remarks
 * `elements` - array of Elements that are the content of the new item
 * `title` - optional string which is used as the new title of the tab (defaults to be the registered name)
 * `closable` - optional boolean which allows the item to be closed with the X button if set and true
 * `size` - optional string describing the size of the new item (see the written documentation for more info)
 * `registration` - optional string configuring the registration ID of the new item (defaults to sequential IDs)
 */
export declare interface RegisteredElementConfig {
    registration: string;
    title?: string;
    closable?: boolean;
    size?: string;
}

/** @internal */
export declare interface RegistrationConfig {
    elements: Element[];
    id?: string;
}

/**
 * @public
 * Versioned layout config objects. `v` is the version and `c` contains the layout config.
 * @remarks
 * Versioning the layout config in this way allows changes to the config schema while
 * remaining backwards compatible with any existing client/user configurations. When loading an
 * older version of the configurations the version can be used to lookup and apply transformer functions
 * to get the newest version of the config from it.
 *
 * As more versions of configurations are added this type will have extra union types as part of SerialisedLayout.
 */
export declare type SerialisedLayout = {
    v: '1';
    c: ResolvedLayoutConfig;
};

export { }
